package org.fouram.job;

import org.fouram.core.plugin.weixin.cp.util.WXCpMessageUtil;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.LoggerUtil;
import org.fouram.core.util.Tools;
import org.fouram.service.SapCheckdateService;
import org.fouram.service.SapEmployeeTimeService;
import org.fouram.service.SapUserService;
import org.fouram.service.WxApprovalInfoService;
import org.fouram.service.WxCheckOptionService;
import org.fouram.service.WxCheckService;
import org.fouram.service.WxDepartService;
import org.fouram.service.WxUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class CheckTaskSchedule {

	@Autowired
	private WxUserService wxUserService;
	@Autowired
	private WxCheckService wxCheckService;
	@Autowired
	private SapUserService sapUserService;
	@Autowired
	private WxDepartService wxDepartService;
	@Autowired
	private SapCheckdateService sapCheckdateService;
	@Autowired
	private WxCheckOptionService wxCheckOptionService;
	@Autowired
	private SapEmployeeTimeService employeeTimeService;
	@Autowired
	private WxApprovalInfoService wxApprovalInfoService;

	public void taskBiz() throws Exception {
		LoggerUtil.info(DateUtil.getTime());
		try {
			if (DateUtil.getHour().equals("10") || DateUtil.getHour().equals("20")) {
				LoggerUtil.info(DateUtil.getDay() + "执行开始:" + DateUtil.getTime());
				executeBiz(DateUtil.getDay());
				LoggerUtil.info(DateUtil.getDay() + "执行结束:" + DateUtil.getTime());
			}
			// 重新执行最近30天
			if (DateUtil.getHour().equals("11")) {
				for (String day : DateUtil.getSubDateList(DateUtil.getAfterDay(-30), DateUtil.getDay())) {
					LoggerUtil.info(day + "执行开始:" + DateUtil.getTime());
					reloadBiz(day);
					LoggerUtil.info(day + "执行结束:" + DateUtil.getTime());
				}
			}
		} catch (Exception e) {
			WXCpMessageUtil.sendAdminError(Tools.getExceptionAllInfo(e));
			LoggerUtil.error(e);
		}
	}

	@Transactional
	public void executeBiz(String checkinDay) throws Exception {
		String startDay = checkinDay;
		String endDay = checkinDay;
		wxDepartService.saveWxDeparts();
		wxUserService.saveWxUsers();
		sapUserService.saveOrUpdateSapUsers();
		wxUserService.updateAllSapUserId();
		wxCheckService.saveCheckInOffOuts(startDay, endDay);
		// 判断打卡记录小于10条记录认为是假期
		int checkInTotal = wxCheckService.queryCheckInTotal().intValue();
		if (checkInTotal < 10) {
			return;
		}
		sapCheckdateService.saveCheckdate(startDay);
		employeeTimeService.saveEmployeeTimes();
		wxCheckOptionService.saveOptions(endDay);
		wxApprovalInfoService.saveApprovalInfos(endDay.substring(0, 7));
		boolean specialCheckinDay = false;
		// 判断小于50条为特殊日期，比如周六有的部分需要上班的，只针对他们生成打卡记录
		if (checkInTotal < 50) {
			specialCheckinDay = true;
		}
		wxCheckService.saveReportResultData(startDay, endDay, specialCheckinDay);
		wxCheckService.updateReportResultData(startDay, endDay);
	}
	
	@Transactional
	public void reloadBiz(String checkinDay) throws Exception {
		wxCheckService.updateReportResultData(checkinDay, checkinDay);
	}
}
